# -*- coding: utf-8 -*-
"""
Created on Mon Aug 23 15:50:10 2021

@author: Michael Nauge (Université de Poitiers)
"""

import requests
import json

from NklResponse import NklResponse


def get_datas(nklTarget, identifier, metadataFormat="--"):
    """
    Récupération des informations sur une donnée.
    Retourne l'ensemble des informations relatives à la donnée
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    metadataFormat : STR
        une valeur de la liste ("--","dc", "qdc")
        
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier
    if (metadataFormat=="qdc") or (metadataFormat=="dc"):
        url+="?metadata-format="+metadataFormat
    
    
    APIheaders = {}
    
    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Retourne l'objet donnée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


    
def put_datas(nklTarget, identifier, dictVals):
    """
    Modification des informations d'une donnée.
    Les informations à modifier doivent être dans le dictionnaire dictVals
    qui sera converti en objet json
    
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala
        
    dictVals : dict
        un dictionnaire à convertir en json compatible avec les clés et valeurs attendues par nakala
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier
    APIheaders = {}

    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        response = requests.put(url, data =json.dumps(dictVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 204 le serveur a bien appliqué la modificatin
        if response.status_code == 204:
            nklR.isSuccess = True
            nklR.message="La donnée a été modifiée"
            # ça ne retourne pas d'objet json en cas de réussite

            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



def delete_datas(nklTarget, identifier):
    """
    La suppression d'une donnée est autorisée uniquement si la donnée n'est pas publiée
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala
    
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        puisqu'il n'y a pas de retour json en cas de réussite 
        NklResponse.dictVals = {}
        
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 204 le serveur a bien supprimé la data
        if response.status_code == 204:
            nklR.isSuccess = True
            nklR.message="La data a été supprimé"
            # Il n'y a pas d'objet json retournée à convertir en dictionnaire python

            
            # on retourne l'objet NklResponse maintenant rempli
            return nklR
        
        else:
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def post_datas(nklTarget, data):
    
    """
    Permet de déposer une donnée dans Nakala
    Les fichiers associés à la donnée sont à déposer avant via POST /uploads
    afin de connaitre leurs fileIdentifier SHA1
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    data : DICT
        une instance d'un dictionaire python (qui sera converti en json) contenant les 
        clé-valeurs compatibles avec nakala
        Pour plus d'informations sur les valeurs possibles à mettre 
        dans le dictionnaire :
            https://apitest.nakala.fr/doc#operations-datas-post_datas
        ex : {
              "status": "published",
              "metas": [
                {
                  "value": "string",
                  "lang": "string",
                  "typeUri": "string",
                  "propertyUri": "string"
                }
              ],
              "files": [
                {
                  "sha1": "string",
                  "description": "string",
                  "embargoed": "string"
                }
              ],
              "collectionsIds": [
                "string"
              ],
              "rights": [
                {
                  "id": "string",
                  "role": "string"
                }
              ]
            }
    
    """
    
    url = nklTarget.API_URL+"/datas"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL,"accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(data), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 201:
            nklR.isSuccess = True
            nklR.message="Enregistrement de la donnée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            # tout c'est bien passé donc on a un 
            # data identifier(DOI) à récupérer dans le json retourné par le server
            # dans nklR.dictVals['payload']['id']
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    
    
def get_datas_files(nklTarget, identifier):
    """
    Permet d'obtenir l'ensemble des informations sur les fichiers associés à une donnée
    
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/files"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Liste des métadonnées des fichiers"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def post_datas_files(nklTarget, identifier, fileInfos):
    """
    Permet d'ajouter un fichier à une donnée.
    Attention, le fichier doit être déposé avant à l'aide de la requête POST /uploads

    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    fileInfos : DICT
        une instance d'un dictionaire python (qui sera converti en json) contenant les 
        clé-valeurs compatibles avec nakala
        Pour plus d'informations sur les valeurs possibles à mettre 
        dans le dictionnaire :
            https://apitest.nakala.fr/doc#operations-datas-post_datas__identifier__files
        ex : {
              "sha1": "string",
              "description": "string",
              "embargoed": "string"
            }
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server
    
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier+"/files"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL,"accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(fileInfos), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Fichier ajouté"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)

            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


def delete_datas_files(nklTarget, identifier, fileIdentifier):
    """
    La suppression d'un fichier ne peut se faire que sur une donnée non publiée
    
    TODO : à débugger car pour le moment la fonction ne semble pas fonctionner...
    

    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    fileIdentifier : STR
        un SHA1 file identifier de la data.


    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server


    """


    url = nklTarget.API_URL+"/datas/"+identifier+"​/files​/"+fileIdentifier
    print(url)
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Fichier supprimé de la donnée"
            # Il n'y a pas d'objet json retournée à convertir en dictionnaire python

            
            # on retourne l'objet NklResponse maintenant rempli
            return nklR
        
        else:
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR    
    

def get_datas_metadatas(nklTarget, identifier, metadataFormat="--"):

    """
    Récupération de la liste des métadonnées d'une donnée
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    metadataFormat : STR
        une valeur de la liste ("--","dc", "qdc")
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/metadatas"
    
    if (metadataFormat=="qdc") or (metadataFormat=="dc"):
        url+="?metadata-format="+metadataFormat

    
    
    print(url)
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Liste des métadonnées des fichiers"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    




def post_datas_metadatas(nklTarget, identifier, dictVals):
    """
    Ajout d'une nouvelle métadonnée à une donnée
    
    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    dictVals : dict
        un dictionnaire à convertir en json compatible avec les clés et valeurs attendues par nakala
        

   Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals
    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/metadatas"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(dictVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 201:
            nklR.isSuccess = True
            nklR.message="La métadonnée a été ajoutée à la donnée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)

            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def delete_datas_metadatas(nklTarget, identifier, dictVals):
    """
    Suppression de métadonnées pour une donnée
    Il est possible de passer un filtre dans le corps de la requête qui permettra de ne supprimer que certaines métadonnées
    Par exemple pour supprimer les dcterms:subject en anglais il faudra passer l'objet suivant:
    {"lang": "en", "propertyUri": "http://purl.org/dc/terms/subject"}
    
    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    dictVals : dict
        un dictionnaire à convertir en json compatible avec les clés et valeurs attendues par nakala
        

   Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        ex: {
              "value": "string",
              "lang": "string",
              "typeUri": "string",
              "propertyUri": "string"
            }
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier+"/metadatas"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, data =json.dumps(dictVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Nombre de métadonnées supprimées"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)

            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR   




def get_datas_rights(nklTarget, identifier):

    """
	Permet de retourner une liste contenant l'utilisateur ou le groupe d'utilisateurs et le droit sur la donnée 
	
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/rights"
    
    
    print(url)
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Liste des utilisateurs et des groupes ayant des droits sur la donnée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    




def post_datas_rights(nklTarget, identifier, dictVals):
    """
    Permet de rajouter des droits à un utilisateur ou un group d'utilisateurs sur une donnée.
	Les droits possibles sont :ROLE_OWNER, ROLE_ADMIN, ROLE_EDITOR, ROLE_READER, 
    
    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    dictVals : dict
        un dictionnaire à convertir en json compatible avec les clés et valeurs attendues par nakala
        ex: [
			  {
				"id": "b55e770c-849b-11ea-87ea-0242ac1b0003",
				"role": "ROLE_READER"
			  }
			]

   Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals
    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/rights"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(dictVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Droits ajoutés sur la donnée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)

            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def delete_datas_rights(nklTarget, identifier, dictVals):
    """
    Suppression des droits pour un utilisateur ou un groupe d'utilisateurs sur une donnée
    La suppression des droits d'une donnée est autorisée aux utilisateurs ayant les droits propriétaire ou administrateur.
	Il est possible de passer un filtre dans le corps de la requête qui permettra de ne supprimer que certains droits.
	Par exemple pour supprimer tous les éditeurs d'une donnée : {"role": "ROLE_EDITOR"}
	Note: le droit propriétaire (ROLE_OWNER) d'une donnée ne peut pas être supprimé.
    
    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    dictVals : dict
        un dictionnaire à convertir en json compatible avec les clés et valeurs attendues par nakala
        

   Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        ex: {
				"id": "b55e770c-849b-11ea-87ea-0242ac1b0003",
				"role": "ROLE_READER"
			}
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier+"/rights"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, data =json.dumps(dictVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Droits supprimés"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)

            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR   



def get_datas_collections(nklTarget, identifier):

    """
	Récupération de la liste des collections contenant la donnée
	
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/collections"
    
    
    print(url)
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="La liste des collections"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    




def post_datas_collections(nklTarget, identifier, listVals):
    """
    Ajout d'une donnée dans un ensemble de collections
    
    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    listVals : list
		Liste des identifiants des collections à associer à la donnée
        
        ex: [
			"10.34847/nkl.12345678",
			"10.34847/nkl.fedcba98"
			]

   Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals
    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/collections"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(listVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 201:
            nklR.isSuccess = True
            nklR.message="La donnée a été ajoutée dans les collections"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)

            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def delete_datas_collections(nklTarget, identifier, listVals):
    """
    Suppression d'une donnée d'un ensemble de collections
    
	L'utilisateur doit au minimum avoir les droits de lecture sur la donnée et être éditeur des différentes collections.
	Ni la donnée, ni les collections ne sont supprimées de NAKALA.

    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
    listVals : LIST
        Liste des identifiants des collections d'où la donnée doit être supprimée
		ex: [
			"10.34847/nkl.12345678",
			"10.34847/nkl.fedcba98"
			]
        

   Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        ex: {
				"id": "b55e770c-849b-11ea-87ea-0242ac1b0003",
				"role": "ROLE_READER"
			}
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier+"/collections"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, data =json.dumps(listVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="La donnée a été supprimé des collections"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)

            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR   




def get_datas_status(nklTarget, identifier):

    """
	Récupération du statut d'une donnée
	Retourne le statut de la donnée. Le statut peut être:
    pending : donnée déposée mais pas encore en ligne
    published : donnée publiée
    deleted : donnée supprimée
    old : ancienne version d'une donnée publiée
	
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala.
        
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server
        Attention en cas de réussite dictVals n'est pas un dictionnaire mais simplement une string contrairement à beaucoup d'autres réponses

    """

    url = nklTarget.API_URL+"/datas/"+identifier+"/status"
    
    
    print(url)
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Statut de la donnée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



    
def put_datas_status(nklTarget, identifier):
    """
    Publication d'une donnée
    Permet de publier une donnée déposée (non encore publique)
    Attention cette opération n'est pas réversible
    
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/datas/"+identifier+"/status/published"
    APIheaders = {}

    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        response = requests.put(url, headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 204 le serveur a bien appliqué la modificatin
        if response.status_code == 204:
            nklR.isSuccess = True
            nklR.message="Status changé"
            # cas de réussite il n'y a rien dans response.txt
            #donc on laisse dictVals vide
            #nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



def get_datas_uploads(nklTarget):
    """
    Les fichiers déposés restent dans un espace temporaire le temps qu'ils soient associés à une donnée de Nakala ou soient automatiquement supprimés (toutes les 24 heures)
    Retourne Liste des objets fichiers déposés
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/datas/uploads"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Liste des objets fichiers déposés"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


def post_datas_uploads(nklTarget, pathFile):
    """
    Permet de déposer un fichier dans un espace temporaire de NAKALA pour 
    être ensuite associé à une donnée (requête POST /datas)
    
    Avant de pouvoir créer une data nakala contenant un fichier
    il faut avoir préalablement envoyé le fichier.
    
    Il faut donc utiliser cette fonction pour envoyer un fichier.
    Une fois le fichier reçu sur nakala, 
    le serveur nous retourne le SHA-1 du fichier.
    Ce fichier est stocké dans un espace temporaire de traitement.
    

    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    pathFile : STR
        le chemin local vers le fichier à envoyer.
    

    

    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server
        Le SHA1 est accessible dans NklResponse.dictVals['sha1']

    """
    
        
    url = nklTarget.API_URL+"/datas/uploads"
    APIheaders = {}
    
    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "accept": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        fileOpened = open(pathFile, "rb")    
        fileCur = {'file': fileOpened}
    
        response = requests.post(url, files=fileCur, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 201:
            nklR.isSuccess = True
            nklR.message="Retourne l'empreinte SHA1 du fichier déposé sur le serveur"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



    
    




def delete_datas_uploads(nklTarget, fileIdentifier):
    """
    Permet de supprimer un fichier présent dans l'espace temporaire
        
    Note : pour supprimer facilement tous les fichiers de l'espace temporaire
    nous recommendons l'utilisation de nklUtils.delete_datas_uploads_all(nklTarget)
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        pusiqu'il n'y a pas de retour json en cas de réussite 
        NklResponse.dictVals = {}
        
    """
    
    url = nklTarget.API_URL+"/datas/uploads/"+fileIdentifier
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Le fichier a été supprimé"
            # Il n'y a pas d'objet json retournée à convertir en dictionnaire python

            
            # on retourne l'objet NklResponse maintenant rempli
            return nklR
        
        else:
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    


def get_iiif_infoJson(nklTarget, identifier, fileIdentifier):
    """
    IIIF Image API - Information sur l'image "fileIdentifier" de la donnée "identifier"
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala
        
    fileIdentifier : STR
        un SHA1 file identifier de la data.

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
   
    
    url = nklTarget.API_URL+"/iiif/"+identifier+"/"+fileIdentifier+"/info.json"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Les informations iiif de l'image"
            try:
                # on converti l'objet json retournée en dictionnaire python
                nklR.dictVals = json.loads(response.text)
            except:
                pass
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            try:
                dicError = json.loads(response.text)
                nklR.message=dicError['message']
            except:
                pass

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


# -*- coding: utf-8 -*-
"""
Created on Fri Jun 11 14:26:26 2021

@author: Michael Nauge, Université de Poitiers
"""
import sys


import pandas as pd

import requests
import json


import nklAPI_Datas as nklD
import nklAPI_Collections as nklC



def collectionToDf(nklTarget, targetCollection):
    
    
    # TODO
    pass
    

def collectionDatasToDf(nklTarget, targetCollection):
    """
    Obtenir toutes les metas des datas d'une collection nakala 
    dont on connait l'identifiant de collection
    

    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget
    
    targetCollection : STR
        un identifier collection nakala.
        ex : '10.34847/nkl.d9b3cp68'

    Returns
    -------
    dfData : DICT
        un dictionnaire contenant les metadatas des datas
        
    dfFile : DICT
        un dictionnaire contenant les metadatas des files des datas

    """   
    
    # exemple d'url de collection à exporter : 
    # https://apitest.nakala.fr/collections/10.34847/nkl.74aa9b31
    
    
    indexPage = 1
    
    lastPage = sys.maxsize

    # creation de la dataframe Datas avec juste les noms de colonnes pour les datas
    dfData = pd.DataFrame(columns=['linkedInCollection','dataIdentifier','uriData','nkl_title','nkl_created','nkl_license','nkl_type', 'nkl_type_converted','creators_formated'])
    
    # creation de la dataframe Files avec juste les noms de colonnes pour les files
    dfFile = pd.DataFrame(columns=['linkedInData','linkedInData_Title','uriData','name','uriFileDL','description','size','extension','sha1','mime_type','embargoed','uriFileEmbed'])
    
    print("start datas collection extraction")

    while indexPage <= lastPage:
        
        
        
        rc = nklC.get_collections_datas(nklTarget, targetCollection, page=indexPage, limit=10)
        indexPage+=1
        
        if rc.isSuccess:
            lastPage = rc.dictVals['lastPage']
            
            print("pagined extraction:", str(indexPage-1)+"/"+str(lastPage))

            
            for data in rc.dictVals['data']:

                dataUri = data["uri"]
                dataUri = dataUri.replace('https://doi.org/','')
                
                print(dataUri,' ', data["status"])
                
                dataTitle = ""
                
                dicRowData = {}
                dicRowData['dataIdentifier'] = dataUri
                dicRowData['uriData'] = nklTarget.BASE_URL+"/"+dataUri
                dicRowData['linkedInCollection'] = "not yet available in nakalapycon"
                
                if 'collectionsIds' in data:
                    print(data["collectionsIds"])
                
                dicRowData['creators_formated'] = ""
                
                for meta in data['metas']:
                    #print(meta['value'], " ", meta['propertyUri'])
                    
                    if meta['propertyUri'] == 'http://nakala.fr/terms#title':
                        #print("title :", meta['value'])
                        dataTitle = meta['value']
                        dicRowData['nkl_title'] = dataTitle
                      
                    if meta['propertyUri'] == 'http://nakala.fr/terms#created':
                        dicRowData['nkl_created'] = meta['value']
                        
                    if meta['propertyUri'] == 'http://nakala.fr/terms#license':
                        dicRowData['nkl_license'] = meta['value']
                      
                    
                    if meta['propertyUri'] == "http://nakala.fr/terms#type":
                        dicRowData['nkl_type'] = meta['value']
                        
                        #dicRowData['nkl_type_converted'] = myConst.vocabTypeGetKeyByUri(meta['value'])  
                        dicRowData['nkl_type_converted'] = ""  
                        
                    if meta['propertyUri'] == "http://nakala.fr/terms#creator":
                        
                        #les anciennes data nakala n'avait pas la meme structuration pour le type creator
                        if not (meta['value']==None):
                            
                            if isinstance(meta['value'], dict):
                                formattedName = meta['value']['givenname']+" "+meta['value']['surname']
                                
                                if dicRowData['creators_formated'] == "":
                                    dicRowData['creators_formated'] = formattedName
                                else:
                                    dicRowData['creators_formated'] += ", "+formattedName
                            else:
                                if dicRowData['creators_formated'] == "":
                                    dicRowData['creators_formated'] = meta['value']
                                else:
                                    dicRowData['creators_formated'] += ", "+meta['value']
                                
                                
                        
                dfData = dfData.append(dicRowData, ignore_index=True) 
                
                for file in data['files']:
                    #print(file['name'], file['extension'],file['size'],file['mime_type'],file['sha1'],file['embargoed'],file['description'])
                    #print(">> uriFile", myConst.API_URL+"/data/"+dataUri+"/"+file['sha1'])
                    dicRowFile = file.copy()
                    #dicRowFile = {}
                    dicRowFile['linkedInData'] = dataUri
                    dicRowFile['linkedInData_Title'] = dataTitle
                    dicRowFile['uriData'] = nklTarget.BASE_URL+"/"+dataUri
                    dicRowFile['uriFileDL'] = nklTarget.API_URL+"/data/"+dataUri+"/"+file['sha1']
                    dicRowFile['uriFileEmbed'] = nklTarget.API_URL+"/embed/"+dataUri+"/"+file['sha1']
                    
                    dfFile = dfFile.append(dicRowFile, ignore_index=True)
                        
        else:
            lastPage = 0

    
    return dfData, dfFile
    

def getImageSize(nklTarget, dataIdentifier, fileIdentifier):
    """
    

    Parameters
    ----------
    nklTarget : OBJ
        une instance d'un objet NklTarget
        
    dataIdentifier : STR
        l'id d'une data qui existe dans le nakala cible
        
        attention : il arrive que l'on cherche une data dans le nakala_production alors 
        qu'elle que cette data n'existe que dans le nakala_test
        
        exemple : dataIdentifier="10.34847/nkl.f1ea3017"
        
    fileIdentifier : STR
        le SHA1 d'un file présent dans la data
        
        exmple : fileIdentifier = "206f92670979917a79a208788e65c2fa4c48634c"

    Returns
    -------
    width : INT
        La largeur d'origine en pixel de l'image cible
    height : INT
       La hauteur d'origine en pixel de l'image cible

    """
    
    width = 0
    height = 0
        
    url = nklTarget.API_URL+"/iiif/"+dataIdentifier+"/"+fileIdentifier+"/info.json"
    
    APIheaders = {}
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL} 

    try :
        response = requests.get(url, headers=APIheaders)
        
        if response.status_code == 200:
            dicoR = json.loads(response.text)

            width = dicoR['width']
            height = dicoR['height']
    except :
        e = sys.exc_info()[0]
        print("error ", e)

    
    return width, height
    
    
    
def getImageUrlIIIF(nklTarget, dataIdentifier, fileIdentifier, region, size, rotation, quality, formatExt):
    """
    Obtenir une URL correctement formaté pour que le serveur IIIF image de 
    Nakala puisse répondre correctement.

    Parameters
    ----------
    nklTarget : OBJ
        une instance d'un objet NklTarget
        
    dataIdentifier : STR
        l'id d'une data qui existe dans le nakala cible
        
        attention : il arrive que l'on cherche une data dans le nakala_production alors 
        qu'elle que cette data n'existe que dans le nakala_test
        
        exemple : dataIdentifier="10.34847/nkl.f1ea3017"
        
    fileIdentifier : STR
        le SHA1 d'un file présent dans la data
        
        exmple : fileIdentifier = "206f92670979917a79a208788e65c2fa4c48634c"
        
    region : STR
        la région de l'image que l'on souhaite
        
        exemple : region="3976,3143,870,618" 
        permet une extraction à partir de l'image d'origine d'un rectangle
        commençant au pixel 3976 sur l'axe horizontal, 
        3143 sur l'axe vertical, 
        de dimension de 870 pixels de largeur et 
        618 pixels de hauteur
    
        
    size : STR
        la taille de l'image cible générée en pixels   
        exemple size="full" ou size="800, 600"
        Si on choisit la valeur full, l'image extraite est fournie à 
        la meilleure taille disponible 
        
    rotation : STR
        correspond à un angle de rotation pour l'image cible 
        (en degrés, dans le sens des aiguilles d'une montre; par exemple "90") 
         exemple : rotation  = "0"
        
        
        
    quality : STR
        correspond à l'espace couleur attendu
        valeurs possible : default, gray, bitonal 
        exemple : quality = "default"
        
    formatExt : STR
        correspond au format de fichier attendu
        valeurs possible : png, jpg, jp2, pdf, gif, tif
        exemple : formatExt="formatExt"

    Returns
    -------
    urlImg : STR
        Une URL correctement formaté pour que le serveur IIIF de Nakala
        puisse répondre correctement

    """
    
    urlImg = nklTarget.API_URL+"/iiif/"+dataIdentifier+"/"+fileIdentifier

    urlImg += "/"+region+"/"+size+"/"+rotation+"/"+quality+"."+formatExt

    return urlImg

        

def getSoundTimeDuration(dataIdentifier, fileIdentifier):
    # TODO 
    
    # vivement un IIIF A/V !
    
    # je tente une estimation par la taille du fichier, un bitrate standard et
    # on dit qu'on est en mono ?!
    
    
    duration = 0
    

    return duration
    


# -*- coding: utf-8 -*-
"""
Created on Tue Aug 24 12:03:13 2021

@author: Michael Nauge (Université de Poitiers)
"""


import requests
import json

from NklResponse import NklResponse

def get_collections(nklTarget, identifier, metadataFormat="--"):
    """
    Récupération des informations sur une collection.
    Retourne l'ensemble des informations relatives à la collection
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un COLLECTION identifier nakala.
        
    metadataFormat : STR
        une valeur de la liste ("--","dc", "qdc")
        
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metacollections obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/collections/"+identifier
    if (metadataFormat=="qdc") or (metadataFormat=="dc"):
        url+="?metadata-format="+metadataFormat
        
    APIheaders = {}
    
    # on gère le cas où la collection est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Informations sur la collection"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


    
def put_collections(nklTarget, identifier, dictVals):
    """
    Modification des informations d'une collection.
    Les informations à modifier doivent être dans le dictionnaire dictVals
    qui sera converti en objet json
    
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala
        
    dictVals : dict
        un dictionnaire à convertir en json compatible avec les clés et valeurs attendues par nakala
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metacollections obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/collections/"+identifier
    APIheaders = {}

    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        response = requests.put(url, data =json.dumps(dictVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 204 le serveur a bien appliqué la modification
        if response.status_code == 204:
            nklR.isSuccess = True
            nklR.message="La collection a été modifiée"
            # ne retourne pas d'objet json en cas de réussite
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



def delete_collections(nklTarget, identifier):
    """
    Suppression d'une collection
	Supprime définitivement la collection.
	Les données contenues dans la collection ne seront pas supprimées
	notes : il est possible aussi de faire simplement quelques modifications avec les fonctions put_collections
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un COLLECTION identifier nakala
    
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        puisqu'il n'y a pas de retour json en cas de réussite 
        NklResponse.dictVals = {}
        
    """
    
    url = nklTarget.API_URL+"/collections/"+identifier
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 204 le serveur a bien supprimé la data
        if response.status_code == 204:
            nklR.isSuccess = True
            nklR.message="La collection a été supprimée"
            # Il n'y a pas d'objet json retournée à convertir en dictionnaire python
            
            # on retourne l'objet NklResponse maintenant rempli
            return nklR
        
        else:
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def post_collections(nklTarget, data):
    """
    Création d'une nouvelle collection
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    data : DICT
        une instance d'un dictionaire python (qui sera converti en json) contenant les 
        clé-valeurs compatibles avec nakala
        Pour plus d'informations sur les valeurs possibles à mettre 
        dans le dictionnaire :
            https://apitest.nakala.fr/doc#operations-collections-post_collections
        ex : {
			  "status": "public",
			  "metas": [
				{
				  "value": "string",
				  "lang": "string",
				  "typeUri": "string",
				  "propertyUri": "string"
				}
			  ],
			  "datas": [
				"string"
			  ],
			  "rights": [
				{
				  "id": "string",
				  "role": "string"
				}
			  ]
			}
    
    """
    
    url = nklTarget.API_URL+"/collections"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL,"accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(data), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 201:
            nklR.isSuccess = True
            nklR.message="La collection a été correctement créée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            # tout c'est bien passé donc on a un 
            # data identifier(DOI) à récupérer dans le json retourné par le server
            # dans nklR.dictVals['payload']['id']
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def get_collections_datas(nklTarget, identifier, page=1, limit=10):
    """
    Récupération de la liste paginée des données contenues dans la collection
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un COLLECTION identifier nakala.
        
    page : INT
        La page souhaitée
    
    limit : INT
        Le nombre de résultat par page
        
        
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metacollections obtenus depuis la reponse json du server

        
    """
    #​/collections​/{identifier}​/datas

    url = nklTarget.API_URL+"/collections/"+identifier+"/datas?page="+str(page)+"&limit="+str(limit)
    

    APIheaders = {}
    
    # on gère le cas où la collection est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Informations sur la collection"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



def post_collections_datas(nklTarget, identifier, datas):
    """
    Ajout d'une liste de données dans une collection  
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un COLLECTION identifier nakala.
    
    datas : Liste des identifiants des données à ajouter à la collection
        ex : [
            "10.34847/nkl.12345678",
            "10.34847/nkl.fedcba98"
            ]
    """
    
    url = nklTarget.API_URL+"/collections/"+identifier+"/datas"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL,"accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(datas), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 201:
            nklR.isSuccess = True
            nklR.message="Les données ont été ajoutées à la collection"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


# -*- coding: utf-8 -*-
"""
Created on Wed May 18 16:16:50 2022

@author: Michael Nauge (Université de Poitiers)

"""


import requests
import json

from NklResponse import NklResponse


def get_search_datas(nklTarget, q="", fq="", facet="", order="relevance"):
    """
    Recherche des données Nakala.
    Retourne des données Nakala en fonction de critères de recherche
    
    Attention, cette fonction retourne également des collections. 
    Mais les colletions trouvées n'ont pas de key files mais une key dataids
    Pour préciser et contraindre à n'obtenir que 
        des datas il faut préciser en paramètre fq = scope=data
        des collections il faut préciser en paramètre fq = scope=collection
    
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    q : STR
    	requête à effectuer
        
    fq : STR
        filtres pour la requête
        Filtres disponibles : scope, status, type, year, created, language, collection, license, fileExt, depositor, owner et share
        Il est possible de rechercher sur plusieurs filtres ; le caractère de séparation est le point-virgule. La recherche est traduite par un ET entre les filtres.
        Il est possible d'ajouter plusieurs valeurs pour un même filtre ; le caratère de séparation est la virgule. La recherche se fera alors par un OU entre les valeurs.
        Exemple : scope=collection;status=public;year=2009,1889
        
        
        
    facet : STR
        facette(s) à retourner
        Facettes disponibles : scope, status, type, year, created, license, language, fileExt, fileSize, fileType et collection
        Il est possible de retrouner plusieurs facettes ; le caractère de séparation est le point-virgule.
        Il est possible de configurer la taille et l'ordre d'une facette avec les paramètres size, sort et order
        Pour la facette created, size correspond au format date souhaité : yyyy, yyyy-MM ou yyyy-MM-dd
        Exemple : type,size=17,sort=item,order=asc;fileExt,size=7,sort=count,order=desc
        
        
    order : STR
         tri des résultats
         Valeurs possibles : relevance ou date,desc ou date,asc ou title,desc ou title,asc
         
        
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/search?"
    
    #ajout des paremtres de filtre
    url += "q="+q
    url += "&fq="+fq
    url += "&facet="+facet
    url + "&order="+order
    
    
    APIheaders = {}
    
    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Retourne une liste de données"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            if 'text' in response:
                dicError = json.loads(response.text)
                nklR.message=dicError['message']
            

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


def get_search_authors(nklTarget, q="", order="asc", page="1", limit="10"):
    """
    Récupération des auteurs associés aux données de Nakala.
    Retourne des auteurs associés aux données de Nakala en fonction de critères de recherche
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    q : STR
    	Mot clé pour la recherche
                
        
    order : STR
        Sens du tri (basé le prénom puis le nom de famille)
        Valeurs possibles : desc ou asc
         
    page : STR
        Page courante
         
    limit : STR
    
        
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/authors/search?"
    
    #ajout des paremtres de filtre
    url += "q="+q
    url + "&order="+order
    url + "&page="+page
    url + "&limit="+limit
    
    
    APIheaders = {}
    
    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Retourne une liste d'auteurs"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



# -*- coding: utf-8 -*-
"""
Created on Mon Aug 23 15:02:00 2021

@author: Michael Nauge (Université de Poitiers)
"""

class NklResponse:
    """
    Une classe (sac de variable) permettant la manipulation homogène 
    des responses serveurs renvoyées et communication reseau.
    On va faire en sorte d'éviter à l'utilisateur d'éviter une gestion d'exception et de message d'erreur
    a deux niveaux (serveur nakala et communication reseau avec request).
    
    """
    
    # le numéro de code renvoyé par le serveur nakala 
    # ou un numéro negatif en cas de problème réseau et/ou avec la librairie request
    code = 0
    # la description associée au code d'erreur nakala
    # ou une exception de la librairie request
    message = ""
    # les valeurs json renvoyés par le serveur en cas de succès sous la forme d'un dictionnaire python
    dictVals = {}
    # un boolean informant de la réussite de la requete
    # c'est pratique quand on sait qu'en fonction des requetes les codes de réussites sont différents
    # 200 pour un get réussi 
    # 204 en cas de modification réussi, 204 en cas de suppression réussie
    # 200 pour un post réussie
    # et aussi que des erreurs peuvent avoir lieux avant le serveur nakala
    # comme des problèmes reseaux ...
    # ça permet de simplifier/uniformiser les appels
    isSuccess = False

    def __init__(self, code=0, message="", dictVals={}, isSuccess=False):
        self.code=code
        self.message=message
        self.dictVals=dictVals
        self.isSuccess=isSuccess
        
        
    def __str__(self):
        return f'nklResponse isSuccess:{self.isSuccess}, code:{self.code},  message:{self.message},  dictVals:{self.dictVals}'
    
        



# -*- coding: utf-8 -*-
"""
Created on Mon Aug 23 15:50:10 2021

@author: Michael Nauge (Université de Poitiers)
"""

import requests
import json

from NklResponse import NklResponse


def get_users_me(nklTarget):
    """
    Récupération des informations sur l'utilisateur courant
    Retourne l'ensemble des informations disponibles sur l'utilisateur courant
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    
        
        
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/users/me"
   
    
    APIheaders = {}
    
    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Informations sur l'utilisateur courant"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


 
 


# -*- coding: utf-8 -*-
"""
Created on Mon Aug 23 15:50:10 2021

@author: Michael Nauge (Université de Poitiers)
"""

import requests
import json

from NklResponse import NklResponse


def search_groups(nklTarget, q, order="asc", page=1, limit=10):
    """
    Récupération des utilisateurs et groupes d'utilisateurs
    Retourne des utilisateurs et groupes d'utilisateurs en fonction de critères de recherche
    
    Parameters
    ----------
    nklTarget : TYPE
        une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    q : STR
        Mot clé pour la recherche
        
    order : STR
        Sens du tri (basé le prénom puis le nom de famille)
        
    page : STR
        Page courante
        
    limit : STR
        Nombre de résultats par page
    
    

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

    """
    
    url = nklTarget.API_URL+"/groups/search"
   
    #https://apitest.nakala.fr/groups/search?q=Huma-Num-test-Admin&order=asc&page=1&limit=10"
    url += "?q=" + q
    url += "&order=" + order
    url += "&page="+str(page)
    url += "&limit="+str(limit)
    
        
    APIheaders = {}
    
    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Retourne l'objet de la recherche"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR    
    
    
        

def get_groups(nklTarget, identifier):
    """
    Récupération des informations d'un groups/liste d'utilistateurs.
    Retourne l'ensemble des informations relatives à ce group
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un groups identifier nakala. 
        exempe : cb5f5980-056e-11ec-9b31-52540084ccd3
        
        
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/groups/"+identifier
   
    
    APIheaders = {}
    
    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.get(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien répondu
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Retourne l'objet donnée"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR


    
def put_groups(nklTarget, identifier, dictVals):
    """
    Le groupe utilisateur transmis dans dictVals remplacera celui déjà existant.
    Note: Il n'est pas possible de modifier des groupes de type "user".
    
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un groups identifier nakala
        
    dictVals : dict
        un dictionnaire à convertir en json compatible avec les clés et valeurs attendues par nakala
        

    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metadatas obtenus depuis la reponse json du server

        
    """
    
    url = nklTarget.API_URL+"/groups/"+identifier
    APIheaders = {}

    # on gère le cas où la data est public
    # et qu'il n'y a donc pas besoin de API_KEY
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL, "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        response = requests.put(url, data =json.dumps(dictVals), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien appliqué la modification
        # surprenant, habituellement ça retour un 204... (mail à huma-num)
        
        #if response.status_code == 204:
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Le group a été modifiée"
            # ça ne retourne pas d'objet json en cas de réussite

            #print("response : ", response)

            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR



def delete_groups(nklTarget, identifier):
    """
    Suppression d'un groupe d'utilisateurs
    La suppression d'un groupe entraine la suppression de tous les droits liés à ce groupe.
    Note: Il n'est pas possible de supprimer un groupe de type "user" ou un groupe propriétaire d'une donnée ou d'une collection
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    identifier : STR
        un DATA identifier nakala
    
    Returns
    -------
    NklResponse : OBJ
        une instance d'un objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        puisqu'il n'y a pas de retour json en cas de réussite 
        NklResponse.dictVals = {}
        
    """
    
    url = nklTarget.API_URL+"/groups/"+identifier
    APIheaders = {}
    
    # on gère le cas où il n'y a pas  de API_KEY
    # ce qui va poser problème pour le serveur nakala
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.delete(url, headers=APIheaders)
        # on récupère le code
        nklR.code = response.status_code
        
        # 200 le serveur a bien supprimé la data
        # habituellement c'est 204 quand le serveur a bien supprimé 
        #if response.status_code == 204:
        if response.status_code == 200:
            nklR.isSuccess = True
            nklR.message="Le group a été supprimé"
            # Il n'y a pas d'objet json retournée à convertir en dictionnaire python
            
            # on retourne l'objet NklResponse maintenant rempli
            return nklR

        else:
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

def post_groups(nklTarget, data):
    """
    Permet de la création d'un nouveau groupe d'utilisateurs
    
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    data : DICT
        une instance d'un dictionaire python (qui sera converti en json) contenant les 
        clé-valeurs compatibles avec nakala
        Pour plus d'informations sur les valeurs possibles à mettre 
        dans le dictionnaire :
            https://apitest.nakala.fr/doc#operations-datas-post_groups
            
        ex :{
              "name": "HUMA-NUM",
              "users": [
                "pdupont",
                "jdubois",
                "lmartin"
              ]
            }
    
    """
    
    url = nklTarget.API_URL+"/groups"
    APIheaders = {}
    
    # on gère le cas où il n'y a pas de API_KEY ce qui posera problème au server
    # mais il nous le fera savoir !
    if nklTarget.apiKey_isEmpty()==False:
        APIheaders = {"X-API-KEY": nklTarget.API_KEY_NKL,"accept": "application/json", "Content-Type": "application/json"}      

    # création d'une instance d'un objet NklResponse à retourner
    nklR = NklResponse()

    try : 
        
        response = requests.post(url, data =json.dumps(data), headers=APIheaders)

        # on récupère le code
        nklR.code = response.status_code
        
        # 201 le serveur a bien répondu
        if response.status_code == 201:
            nklR.isSuccess = True
            nklR.message="Enregistrement du groupe"
            # on converti l'objet json retournée en dictionnaire python
            nklR.dictVals = json.loads(response.text)
            # tout c'est bien passé donc on a un 
            # le group identifier à récupérer dans le json retourné par le server
            # dans nklR.dictVals['payload']['id']
            # on retourne l'objet NklResponse maintenant entièrement rempli
            return nklR
        
        else:
            
            dicError = json.loads(response.text)
            nklR.message=dicError['message']

        
    except requests.exceptions.RequestException as e:
        nklR.code=-1
        nklR.message=e
        
    #on retourne l'objet NklResponse avec erreur (de nakala ou de connexion reseau)
    return nklR
    

 
 


# -*- coding: utf-8 -*-
"""
Created on Thu Jul 22 11:56:22 2021

@author: Michael Nauge, Université de Poitiers


Les codes de communication avec Nakala sont dispersés dans différents
fichiers mais l'idée est que celà reste transparent 
pour l'utilisateur de la librairie
"""

from NklTarget import *
from NklResponse import *

from nklAPI_Collections import *
from nklAPI_Datas import *
from nklAPI_Groups import *
from nklAPI_Users import *
from nklAPI_Vocabularies import *
from nklAPI_Search import *

from nklUtils import *
from nklPullCorpus import *


# -*- coding: utf-8 -*-
"""
Created on Tue Jan  5 10:17:57 2021

@author: Michael Nauge, Université de Poitiers
"""



# un dictionnaire permettant de faciliter l'attribution 
# pour des humains d'un data_type
# nous conseillons de passer par la fonction vocabTypeGetUriByKey(key)
# plutôt que de l'utiliser directement
VOCABTYPE = {
    
    # types classiques (dublincore)
    #-------------------------------------------------
    "text":"http://purl.org/coar/resource_type/c_18cf",
    "image":"http://purl.org/coar/resource_type/c_c513",
    "video":"http://purl.org/coar/resource_type/c_12ce",
    "sound":"http://purl.org/coar/resource_type/c_18cc",
    
    # Attention : lorsque l'on dépose un fichier JPG contenant un texte manuscrit ou tapuscrit
    # il faut choisir le type text plutôt que le type image    
    # c'est probablement là le principal intérêt de préciser cette information 
    # à chaque data nakala car en général l'extension du fichier suffit
    # à connaitre le type
    #-------------------------------------------------
    
    
    # types fils des types classiques ou plus précis ou rares
    # mais dont l'utilisation ne fait pas toujours concensus dans certaines
    # communautés disciplinaires
    #-------------------------------------------------
    "journal article":"http://purl.org/coar/resource_type/c_6501",
    "conference poster":"http://purl.org/coar/resource_type/c_6670",
    "conference object":"http://purl.org/coar/resource_type/c_c94f",
    "learning object":"http://purl.org/coar/resource_type/c_e059",
    "book":"http://purl.org/coar/resource_type/c_2f33",
    "map":"http://purl.org/coar/resource_type/c_12cd",
    "dataset":"http://purl.org/coar/resource_type/c_ddb1",
    "software":"http://purl.org/coar/resource_type/c_5ce6",
    "other":"http://purl.org/coar/resource_type/c_1843",
    "ArchiveMaterial":"http://purl.org/library/ArchiveMaterial",
    "Collection":"http://purl.org/ontology/bibo/Collection",
    "bibliography":"http://purl.org/coar/resource_type/c_86bc",
    "Series":"http://purl.org/ontology/bibo/Series",
    "book review":"http://purl.org/coar/resource_type/c_ba08",
    "manuscript":"http://purl.org/coar/resource_type/c_0040",
    "letter":"http://purl.org/coar/resource_type/c_0857", # A brief description of important new research, also known as “communication”. 
    "report":"http://purl.org/coar/resource_type/c_93fc",
    "periodical":"http://purl.org/coar/resource_type/c_2659", # This concept is deprecated 
    "preprint":"http://purl.org/coar/resource_type/c_816b",
    "review":"http://purl.org/coar/resource_type/c_efa0",
    "musical notation":"http://purl.org/coar/resource_type/c_18cw",
    "SurveyDataSet":"https://w3id.org/survey-ontology#SurveyDataSet",    
    "thesis":"http://purl.org/coar/resource_type/c_46ec",
    "website":"http://purl.org/coar/resource_type/c_7ad9",
    "data paper":"http://purl.org/coar/resource_type/c_beb9",
    "interactive resource":"http://purl.org/coar/resource_type/c_e9a0"
    }




def vocabTypeGetUriByKey(key):
    """
    Obtenir l'uri associé à un nom de concept
    exemple: vocabTypeGetUriByKey(key="image")
    Doit retourner : "http://purl.org/coar/resource_type/c_c513"
    
    Nous conseillons d'utiliser cette fonction 
    plutôt que directement le dictionnaire VOCABTYPE car 
    cett fonction se charge de vérifier l'existance de la key passé en paramètre
    et évite une levée d'exception bloquante. Dans ce cas l'uri retourné sera
    "unknown by nakalapycon or nakala"
    

    Parameters
    key : str
    	une clé 
        
    Returns
    -------
    uri : str
        l'Uri d'un concept datatype reconnu par nakala,
        si la clé f
        

    """   
    
    value = "unknown by nakalapycon"
    
    if key in VOCABTYPE:
        return VOCABTYPE[key]
    
    return value




def vocabTypeGetKeyByUri(uri):
    """
    Obtenir un version compréhensible par un humain du concept
    qui se cache derrière une uri de concept nakala
    
    

    Parameters
    ----------
    uri : str
        une uri présente dans VOCABTYPE

    Returns
    -------
    key : str
        le preflabel du concept associé à l'uri

    """
    
    value = "unknown by nakalapycon"
 
    keys = [k for k, v in VOCABTYPE.items() if v == uri]
    
    
    if len(keys)==1:
        return keys[0]

    return value

    


# -*- coding: utf-8 -*-
"""
Created on Fri Jul 16 09:28:54 2021

@author: mnauge
"""

class NklTarget:
    """
    Une classe permetant de définir le Nakala cible.
    En effet,  il existe :
        Nakala_production (https://nakala.fr)
        Nakala_test (https://test.nakala.fr)
        
    Pour cette cible Nkala, il est possible d'indiquer sa ApiKey pour faire
    des actions qui necessitent des droits.
    
    """
    
    # les urls de Nakala_test
    BASE_URL = "https://test.nakala.fr"
    API_URL = "https://apitest.nakala.fr"
    #la ApiKey
    API_KEY_NKL = ""

    def __init__(self, isNakalaProd=False, apiKey = "01234567-89ab-cdef-0123-456789abcdef"):
        
        # par défaut on considère que l'on travail sur la version de test.
        # si on travail sur la version de production
        # il faut changer les URL
        if isNakalaProd:
            # les urls de Nakala_production
            self.BASE_URL = "https://nakala.fr"
            self.API_URL = "https://api.nakala.fr"
        
        # on met à jour la ApiKey avec la valeur donnée en paramètre entrant
        self.API_KEY_NKL = apiKey
            
        
    def apiKey_isEmpty(self):
        """
        Savoir si la ApiKey a été renseigné.
        Seul une partie des actions Get sur données public peuvent s'effectuer
        sans ApiKey. 
            
        Returns
        -------
        dfData : BOOL

    
        """   
        if self.API_KEY_NKL == "":
            return True
        else:
            return False
        



# coding: utf-8
import setuptools

VERSION = "0.0.6"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nakalapycon",                     # This is the name of the package
    version=VERSION,                        # The initial release version
    author="Michael Nauge",        # Full name of the author
    url="https://gitlab.huma-num.fr/mshs-poitiers/plateforme/nakalapycon",
    project_urls={
        "Issues": "https://gitlab.huma-num.fr/mshs-poitiers/plateforme/nakalapycon/-/issues",
        "CI": "https://gitlab.huma-num.fr/mshs-poitiers/plateforme/nakalapycon/-/pipelines",
        "Changelog": "https://gitlab.huma-num.fr/mshs-poitiers/plateforme/nakalapycon/-/blob/master/CHANGELOG.md",
    },
    description="Librairie Python pour interagir avec Nakala (Nakala est un entrepôt de données de recherche en SHS développé par Huma-Num)",
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),    # List of all python modules to be installed
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.6',                # Minimum version requirement of the package
    py_modules=["nakalapycon","constantes","NklTarget","NklResponse","nklAPI_Collections","nklAPI_Datas","nklAPI_Groups","nklAPI_Users","nklAPI_Vocabularies","nklAPI_Search","nklUtils","nklPullCorpus"],             # Name of the modules inside the python package
    package_dir={'':'nakalapycon/src'},     # Directory of the source code of the package
    install_requires=['requests', 'pandas']           # Install other dependencies if any
)



# -*- coding: utf-8 -*-
"""
Created on Tue Aug 24 11:46:33 2021

@author: Michael Nauge (Université de Poitiers)
"""
import sys

import NklTarget as nklT
import nklAPI_Datas as nklD
import nklAPI_Collections as nklC


def delete_datas_uploads_all(nklTarget):
    """
    Permet de supprimer tous les fichiers présent dans l'espace temporaire
        
    Parameters
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test

    Returns
    -------
    List(NklResponse) : List(OBJ)
        une liste contenant des instances d'objet NklResponse
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        pusiqu'il n'y a pas de retour json en cas de réussite 
        NklResponse.dictVals = {}

        
    """
    
    # on récupère la list des sha1 des fichiers de l'espace temporaire
    r = nklD.get_datas_uploads(nklTarget)
    
    listNklR = []
    
    for dicFile in r.dictVals:
        
        fileIdentifier = dicFile['sha1']
        
        #print("lancer suppression de", fileIdentifier) 
        rd = nklD.delete_datas_uploads(nklTarget, fileIdentifier)
        listNklR.append(rd)
        
    return listNklR
    

def put_collections_datas_rights(nklTarget, collectionId, userGroupId, role, checkOnly=True):
    """
    Permet d'ajouter des droits à un utilisateur ou un group d'utilisateur
    des droits (role) à toutes les datas d'une collection cible
    
    Pour currentData in listDesDatasDeLacollection:
        doi =  obtenir le DOI de la currentData
        
        si la data possède déjà le userGroupId avec le bon role
            pas besoin de modifier cette data
        
        sinon
            ajouter le userGroupId avec le bon role
    

    Parameters
    ----------
    nklTarget : OBJ
    	une instance d'un objet NklTarget permettant de choisir nakala_prod ou nakala_test
        
    collectionId : STR
        un COLLECTION identifier nakala.

    userGroupId : STR
        un USER ou USERGROUP identifier nakala.
        
    role : STR
        valeurs possibles :
            administrateur (ROLE_ADMIN) : consultation, modification, suppression, partage des droits de la collection
            éditeur (ROLE_EDITOR) : consultation, modification de la collection
            lecteur (ROLE_READER) : consultation de la collection même si elle est privée
            
    checkOnly : BOOL
        permet simplement de faire des affichage sans envoyer réellement les 
        requêtes d'ajouts de droit


    Returns
    -------

    NklResponse : List(OBJ)
        une liste d'objets NklResponse (un objet NklResponse par datade la collection cible)
        - en cas de problème reseau ou de message d'erreur du serveur nakala
        l'objet NklResponse.isSuccess=False et le détails de l'erreur en version textuel
        sera dans NklResponse.message
        
        - en cas de réussite de la requete
        l'objet NklResponse.isSuccess=True et 
        les métadonnées reçu seront dans
        NklResponse.dictVals : un dictionnaire contenant les metacollections obtenus depuis la reponse json du server


    """
    
    indexPage = 1
    lastPage = sys.maxsize
    
    listResponses = []
    
    while indexPage <= lastPage:
        rc = nklC.get_collections_datas(nklTarget, collectionId, page=indexPage, limit=1)
        indexPage+=1
        
        if rc.isSuccess:
            lastPage = rc.dictVals['lastPage']
            
            for currentData in rc.dictVals['data']:
                #obtenir le DOI de la current data
                doi = currentData['identifier']
                print("data Identifier:", doi)
                
                # verifier les rights existans de la data
                rd = nklD.get_datas_rights(nklTarget, doi)
                
                needAddRights = True
                
                if rd.isSuccess:
                    #print(rd.dictVals)
                    for right in rd.dictVals:
                        print(right['name'],right['role'])
                        
                        # verifie si le right actuel est celui attendu
                        if (userGroupId == right['id']) & (role == right['role']):
                            needAddRights = False
                            
                    # si on pas pas trouvé le rights attendu pour le usergroup cible
                    # il faut l'ajouter
                    if needAddRights:
                        print(">> add rights needed")
                        
                        if checkOnly==False:
                            print(">> >> send post_datas_rights request")
                            listDicRights=[{"id": userGroupId,"role": role}]
                            rr = nklD.post_datas_rights(nklTarget, doi, listDicRights)
                            listResponses.append(rr)
                    else:
                        print(">> do not need add rights")
                    
        else:
            listResponses.append(rc)
            lastPage = 0
            
            
    return listResponses
            


def isFileNameInData(nklResp, filename):
    """
    vérifier la présence d'un name dans les files d'une data nakala
    probablement un jour obsolete si ajout de cette interrogation dans l'api nakala
    
    Parameters
    ----------
    nklResp : OBJ nklResponse
        un Objet nklResponse (généralement obtenu par un appel à get_search_datas)
        on traite ici lecas particulier où il y a une seule data dans nklResponse
        

    Returns
    -------
    isFile : Bool
        retourne True si le filename est présent dans la data obtenu dans le nklRespone
        False sinon
    shaA : STR
        retourne le sha1 (que dans le cas où isFile==True)

    """
            
    data  = nklResp.dictVals
    
    if "files" in data:
        for file in data["files"]:
            if file["name"]==filename:
                return True, file['sha1']

    
    else:
        pass
    
    return False,""

    
def isFileSha1InData(nklResp, sha1):
    """
    vérifier la présence d'un sha1 dans les files d'une data nakala
    probablement un jour obsolete si ajout de cette interrogation dans l'api nakala
    
    Parameters
    ----------
    nklResp : OBJ nklResponse
        un Objet nklResponse (généralement obtenu par un appel à get_search_datas)
        on traite ici lecas particulier où il y a une seule data dans nklResponse
        

    Returns
    -------
    isFile : Bool
        retourne True si le filename est présent dans la data obtenu dans le nklRespone
        False sinon

    """
            
    data  = nklResp.dictVals
        
    if "files" in data:
        for file in data["files"]:
            if file["sha1"]==sha1:
                return True, file['sha1']
    
    else:
        return False, "" 
    
    
def isFileNameInUploads(nklResp, filename):
    """
    vérifier la présence d'un filename dans la liste des fichier "temporaire" uploads sur nakala avant agrégation par une data
    probablement un jour obsolete si ajout de cette interrogation dans l'api nakala
    
    Parameters
    ----------
    nklResp : OBJ nklResponse
        un Objet nklResponse (généralement obtenu par un appel à get_datas_uploads)
        

    Returns
    -------
    isFile : Bool
        retourne un tuple True + le sha1 si le filename est présent dans la data obtenu dans le nklRespone 
        sinon retourne False + empty string

    """
    
    listDico = nklResp.dictVals
    
    for kv in listDico:
        if kv['name']==filename:
            return True, kv['sha1']
    else:
        return False, ""
        
        

def isFileSha1InUploads(nklResp, filename):
    """
    vérifier la présence d'un sha1 dans la liste des fichier "temporaire" uploads sur nakala avant agrégation par une data
    probablement un jour obsolete si ajout de cette interrogation dans l'api nakala
    
    Parameters
    ----------
    nklResp : OBJ nklResponse
        un Objet nklResponse (généralement obtenu par un appel à get_datas_uploads)
        

    Returns
    -------
    isFile : Bool
        retourne True si le filename est présent dans la data obtenu dans le nklRespone
        False sinon

    """
    
    listDico = nklResp.dictVals
    
    for kv in listDico:
        if kv['sha1']==filename:
            return True
    else:
        return False
            
    